% A sample case for the use PCA-based transform

% load the image
I = readim;

% prepare the variables
edge_size=8;
block_size = edge_size*edge_size;
image_size = size(I,1)*size(I,2);
image_width = size(I,1);
image_height = size(I,2);
B = zeros(block_size, image_size/block_size);
V = zeros(block_size, image_size/block_size);
J = newim(image_width, image_height);
Bnew = zeros(block_size, image_size/block_size);
num_of_B = image_size/block_size;
clear mask_of_components;

% Crop the rectangular blcks B from the image.
% All the B are expected to be of the same size.
idx = 0;

while (idx*block_size < image_size)
    pos_x = mod(idx, image_width/edge_size) * edge_size;
    pos_y = floor(idx / (image_width/edge_size)) * edge_size;    
    crop = I(pos_x:pos_x+edge_size-1, pos_y:pos_y+edge_size-1);
    %block{idx+1} = crop;
    B(:,idx+1) = crop(:);    
    idx = idx+1;
end

% Substract the mean block B 
mean_block = mean(B');
for i=1:num_of_B
    V(:,i) = B(:,i) - mean_block';
end

% Find the covariance matrix    
C_V = V*V';

% Find eigenvalues and eigenvectors.
[A,C_W]=eig(C_V);

% We must transpose A as in Matlab the eigenvectors form the columns of A
A = A';

% apply transform 
W = A*V;

% clear the less important component
keep = 0.0003;
for pos=1:block_size
    mask_of_components(pos) = C_W(pos,pos) >= keep*sum(C_W(:));
end

% show the importance of individual components
components = diag(C_W)'

% show which components are keep compared to the others
mask_of_components

% keep only the selected
for j=1:block_size
    W(j,:)=W(j,:)*mask_of_components(j);   
end

% make inverse transform
Vnew = A'*W;

% add mean
for i=1:num_of_B
    Bnew(:,i) = floor(Vnew(:,i) + mean_block');
end

% form the original image
idx = 0;

while (idx*block_size < image_size)
    pos_x = mod(idx, image_width/edge_size) * edge_size;
    pos_y = floor(idx / (image_width/edge_size)) * edge_size;    
    J(pos_x:pos_x+edge_size-1, pos_y:pos_y+edge_size-1) = reshape(Bnew(:, idx+1), edge_size, edge_size);    
    idx = idx+1;
end

% show the results
result = dip_image(J)

% shot the difference
stretch(result - I)
